/******************************************************************************
* File Name:   main.c
*
* Description: This is the source code for the Empty PSoC6 Application
*              for ModusToolbox.
*
* Related Document: See README.md
*
*
*******************************************************************************
* (c) 2019-2021, Cypress Semiconductor Corporation. All rights reserved.
*******************************************************************************
* This software, including source code, documentation and related materials
* ("Software"), is owned by Cypress Semiconductor Corporation or one of its
* subsidiaries ("Cypress") and is protected by and subject to worldwide patent
* protection (United States and foreign), United States copyright laws and
* international treaty provisions. Therefore, you may use this Software only
* as provided in the license agreement accompanying the software package from
* which you obtained this Software ("EULA").
*
* If no EULA applies, Cypress hereby grants you a personal, non-exclusive,
* non-transferable license to copy, modify, and compile the Software source
* code solely for use in connection with Cypress's integrated circuit products.
* Any reproduction, modification, translation, compilation, or representation
* of this Software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress
* reserves the right to make changes to the Software without notice. Cypress
* does not assume any liability arising out of the application or use of the
* Software or any product or circuit described in the Software. Cypress does
* not authorize its products for use in any products where a malfunction or
* failure of the Cypress product may reasonably be expected to result in
* significant property damage, injury or death ("High Risk Product"). By
* including Cypress's product in a High Risk Product, the manufacturer of such
* system or application assumes all risk of such use and in doing so agrees to
* indemnify Cypress against all liability.
*******************************************************************************/

#include "cy_pdl.h"
#include "cyhal.h"
#include "cybsp.h"
#include "cy_retarget_io.h"

#define ARD_CH_NUM		6
#define TIMER_PERIOD	9999
#define TIMER_CLK_HZ	100000

void handle_error(void);
cy_rslt_t app_hw_init(void);
static void isr_timer(void *callback_arg, cyhal_timer_event_t event);

/* ADC Object */
cyhal_adc_t adc_obj;

/* ADC Channel Objects */
cyhal_adc_channel_t adc_channels[ARD_CH_NUM];

/* Timer Object */
cyhal_timer_t timer_obj;

/* Default ADC configuration */
const cyhal_adc_config_t adc_config =
{
        .continuous_scanning=true,  // Continuous Scanning is enabled
        .average_count=1,           // Average count disabled
        .vref=CYHAL_ADC_REF_VDDA,   // VREF for Single ended channel set to VDDA
        .vneg=CYHAL_ADC_VNEG_VSSA,  // VNEG for Single ended channel set to VSSA
        .resolution = 12u,          // 12-bit resolution
        .ext_vref = NC,             // No connection
        .bypass_pin = NC
};

/* ADC channel configuration */
const cyhal_adc_channel_config_t channel_config =
{
		.enable_averaging = false,  // Disable averaging for channel
        .min_acquisition_ns = 1000, // Minimum acquisition time set to 1us
        .enabled = true
};

/*Default Timer configuration */
const cyhal_timer_cfg_t timer_cfg =
{
    .compare_value = 0,                 /* Timer compare value, not used */
    .period = TIMER_PERIOD,   					/* Defines the timer period */
    .direction = CYHAL_TIMER_DIR_UP,    /* Timer counts up */
    .is_compare = false,                /* Don't use compare mode */
    .is_continuous = true,              /* Run timer indefinitely */
    .value = 0                          /* Initial value of counter */
};

/* ADC data storage */
int32_t adc_data[ARD_CH_NUM] = {0};

int main(void)
{
    cy_rslt_t result;

    /* Initialize the device and board peripherals */
    result = cybsp_init() ;
    if (result != CY_RSLT_SUCCESS)
    {
    	handle_error();
    }

    /*Enable debug output via KitProg UART*/
    result = cy_retarget_io_init( CYBSP_DEBUG_UART_TX, CYBSP_DEBUG_UART_RX, CY_RETARGET_IO_BAUDRATE);
    if (result != CY_RSLT_SUCCESS)
    {
        handle_error();
    }
    printf("\x1b[2J\x1b[;H");
    printf("CY8CKIT-062S2-43012 Arduino ADC DMA HAL Example.\r\n");
    printf("Sample rate: %uHz\r\n", (TIMER_CLK_HZ-1)/TIMER_PERIOD);

    /*Initialize LEDs*/
    result = cyhal_gpio_init( CYBSP_LED_RGB_GREEN, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, CYBSP_LED_STATE_OFF);
    if (result != CY_RSLT_SUCCESS)
    {handle_error();}
    result = cyhal_gpio_init( CYBSP_LED_RGB_RED, CYHAL_GPIO_DIR_OUTPUT, CYHAL_GPIO_DRIVE_STRONG, CYBSP_LED_STATE_OFF);
    if (result != CY_RSLT_SUCCESS)
    {handle_error();}

    __enable_irq();

    /*Configure the hardware*/
    result = app_hw_init();
    if (result != CY_RSLT_SUCCESS)
    {
        handle_error();
    }

    for (;;)
    {
    	CyDelay(1000);
    	printf("A0: %4ld, A1: %4ld, A2: %4ld, A3: %4ld, A4: %4ld, A5: %4ld\r\n", adc_data[0], adc_data[1], adc_data[2], adc_data[3], adc_data[4], adc_data[5]);
    }
}

void handle_error(void)
{
     /* Disable all interrupts. */
    __disable_irq();

    CY_ASSERT(0);
}

cy_rslt_t app_hw_init(void)
{
	cy_rslt_t result;

	/* Initialize the Arduino ADC Block.*/
	cyhal_adc_free(&adc_obj);
	result = cyhal_adc_init(&adc_obj, CYBSP_A0, NULL);
    if(result != CY_RSLT_SUCCESS)
    {
    	goto return_err;
    }

    /* Initialize a channel A0 and configure it to scan P10_0 in single ended mode. */
    result  = cyhal_adc_channel_init_diff(&adc_channels[0], &adc_obj, CYBSP_A0, CYHAL_ADC_VNEG, &channel_config);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /* Initialize a channel A1 and configure it to scan P10_1 in single ended mode. */
    result  = cyhal_adc_channel_init_diff(&adc_channels[1], &adc_obj, CYBSP_A1, CYHAL_ADC_VNEG, &channel_config);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /* Initialize a channel A2 and configure it to scan P10_2 in single ended mode. */
    result  = cyhal_adc_channel_init_diff(&adc_channels[2], &adc_obj, CYBSP_A2, CYHAL_ADC_VNEG, &channel_config);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /* Initialize a channel A3 and configure it to scan P10_3 in single ended mode. */
    result  = cyhal_adc_channel_init_diff(&adc_channels[3], &adc_obj, CYBSP_A3, CYHAL_ADC_VNEG, &channel_config);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /* Initialize a channel A4 and configure it to scan P10_4 in single ended mode. */
    result  = cyhal_adc_channel_init_diff(&adc_channels[4], &adc_obj, CYBSP_A4, CYHAL_ADC_VNEG, &channel_config);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /* Initialize a channel A5 and configure it to scan P10_5 in single ended mode. */
    result  = cyhal_adc_channel_init_diff(&adc_channels[5], &adc_obj, CYBSP_A5, CYHAL_ADC_VNEG, &channel_config);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /* Set-up Mode */
    result = cyhal_adc_set_async_mode(&adc_obj, CYHAL_ASYNC_DMA, CYHAL_DMA_PRIORITY_DEFAULT);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /* Update ADC configuration */
    result = cyhal_adc_configure(&adc_obj, &adc_config);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /* Initialize the Timer */
    result = cyhal_timer_init(&timer_obj, NC, NULL);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /* Update Timer configuration */
    result = cyhal_timer_configure(&timer_obj, &timer_cfg);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /* Set the frequency of Timer's clock source */
    result = cyhal_timer_set_frequency(&timer_obj, TIMER_CLK_HZ);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

    /*Register the callback*/
    cyhal_timer_register_callback(&timer_obj, isr_timer, NULL);

    /*Enable Timer Interrupt*/
    cyhal_timer_enable_event(&timer_obj, CYHAL_TIMER_IRQ_TERMINAL_COUNT,  3, true);

    /*Start the Timer*/
    result = cyhal_timer_start(&timer_obj);
    if(result != CY_RSLT_SUCCESS)
    {goto return_err;}

	return_err:
	return result;
}

static void isr_timer(void *callback_arg, cyhal_timer_event_t event)
{
	cy_rslt_t result;

	/*Transfer ADC data using DMA*/
    result = cyhal_adc_read_async(&adc_obj, ARD_CH_NUM, adc_data);
    if(result != CY_RSLT_SUCCESS)
    {
    	handle_error();
    }

}

/* [] END OF FILE */
